import json

from click import echo
from click import style

from .__version__ import VERSION


def check_unicode_support():
    try:
        echo(
            style(
                text="✔️ Unicode Supported",
                fg="yellow",
            ),
        )
        return True
    except UnicodeEncodeError:
        echo(
            style(
                text="Unicode Unsupported",
                fg="yellow",
            ),
        )
        return False


def initialize_config(path):
    with open(path, "w") as file:
        config = {
            "theme": "light",
            "default_output": "table",
            "unicode": check_unicode_support(),
            "version": VERSION,
            "pretty_tree": True,
        }
        json.dump(config, file, indent=4)
        return config


def update_config(path, config):
    with open(path, "w") as file:
        json.dump(config, file, indent=4)


def get_config(path):
    with open(path, "r") as file:
        return json.load(file)



import json

import click
import keyring
import requests
from requests.auth import HTTPBasicAuth

import jira.database
from app.application import add_tasks
from app.application import update_task


def update_issues(url, email):
    new_issues = fetch_issues(url, email)
    for issue in parse_issues(new_issues):
        old_issue = jira.database.search_task(issue["title"])
        parent = jira.database.search_task(issue["parent"])
        if old_issue:
            old_issue["title"] = issue["title"]
            old_issue["priority"] = issue["priority"]
            old_issue["label"] = issue["label"]
            old_issue["parent_id"] = parent.get("id", None)
            old_issue["status"] = issue["status"]
            update_task(old_issue)

        else:
            add_tasks(
                title=issue["title"],
                priority=issue["priority"],
                description=issue["description"],
                label=issue["label"],
                parent=parent,
                inprogress=(issue["status"] == "In Progress"),
                pending=(issue["status"] == "Pending"),
                completed=(issue["status"] == "Completed"),
            )


def parse_issues(issues):
    new_issues = []
    for issue in issues:
        issue_details = {
            "title": issue["key"],
            "description": issue["fields"]["summary"],
            "priority": int(issue["fields"]["priority"]["id"]),
            "label": "Jira/" + issue["fields"]["issuetype"]["name"],
            "parent": issue["fields"].get("parent", {}).get("key", ""),
        }
        if issue["fields"]["status"]["name"] == "Open":
            issue_details["status"] = "Pending"
        elif issue["fields"]["status"]["name"] == "In Progress":
            issue_details["status"] = "In Progress"
        else:
            issue_details["status"] = "Completed"
        new_issues.append(issue_details)
    return new_issues


def fetch_issues(url, email) -> list:
    token = keyring.get_password("devcord", "token")
    if token is None:
        click.echo(
            click.style(
                'Error: Please provide token, run "devcord jira --token".',
                fg="red",
            ),
        )
        exit(1)
    headers = {
        "Accept": "application/json",
        "Content-Type": "application/json",
    }
    auth = HTTPBasicAuth(email, token)
    try:
        response = requests.get(
            url
            + "rest/api/2/search?jql=assignee=currentUser()&fields=summary,status,priority,issuetype,parent",
            headers=headers,
            auth=auth,
        )
    except Exception as e:
        click.echo(
            click.style(
                f"Error: While making request to Jira - {e}",
                fg="red",
            ),
        )
        exit(1)
    if "application/json" in response.headers.get("Content-Type", ""):
        issues = json.loads(response.text)  # Parse JSON

    else:
        click.echo(
            click.style(
                "Error: Respone from Jira was in-correct. Make sure the token is correct.",
                fg="red",
            ),
        )
        exit(1)
    return issues["issues"]



import sqlite3

from app.constants import db_path


def initialize():
    """
    Initialize the database with all the necessary tables.
    """
    conn = sqlite3.connect(db_path)
    cur = conn.cursor()
    cur.execute(
        """CREATE TABLE tasks(
        id INTEGER PRIMARY KEY AUTOINCREMENT,
        title VARCHAR NOT NULL, parent_id INTEGER,
        description TEXT DEFAULT 'None',
        status VARCHAR DEFAULT 'Pending',
        deadline DATE DEFAULT 'None',
        priority INTEGER DEFAULT 0,
        label VARCHAR DEFAULT 'None',
        completed DATE,
        subtasks INTEGER DEFAULT 0
        )""",
    )
    cur.execute(
        """
        CREATE TRIGGER initialize_completed_column
        AFTER INSERT ON tasks
        FOR EACH ROW
        BEGIN
            UPDATE tasks SET completed = NEW.deadline WHERE id = NEW.id;
        END;
    """,
    )


def list_table(
    table: str,
    columns: list,
    where_clause: str = "",
    group_clause: str = "",
    order_by: str = "",
) -> list:
    """
    List data from a table.
    """
    query = f"SELECT {', '.join(columns)} FROM {table} {where_clause} {group_clause} {order_by}"
    conn = sqlite3.connect(db_path)
    cur = conn.cursor()
    res = cur.execute(query).fetchall()
    return res


def insert_into_table(table: str, columns: list, values: list) -> None:
    """
    Insert data into a table.
    """
    query = f"INSERT INTO {table} ({', '.join(columns)}) VALUES ({', '.join(values)})"
    conn = sqlite3.connect(db_path)
    cur = conn.cursor()
    cur.execute(query)
    conn.commit()


def update_table(table: str, new_data: dict) -> None:
    """
    Update Values Of Given Table
    """
    set_clause = ", ".join(
        [f"{key} = {value}" for key, value in new_data.items() if key != "id"],
    )
    query = f"UPDATE {table} SET {set_clause} WHERE id = {new_data['id']}"
    conn = sqlite3.connect(db_path)
    cur = conn.cursor()
    cur.execute(query)
    conn.commit()


def delete_task(task_id: int) -> None:
    """
    Delete Task From Database
    """
    query = f"DELETE FROM tasks WHERE id = {task_id}"
    conn = sqlite3.connect(db_path)
    cur = conn.cursor()
    cur.execute(query)
    conn.commit()



import os

from click import echo
from click import style


def get_priority_color(priority):
    if priority == 5:
        return "bold red"
    elif priority == 4:
        return "#EE4B2B"
    elif priority == 3:
        return "magenta"
    elif priority == 2:
        return "blue"
    elif priority == 1:
        return "cyan"
    else:
        return "#FFFFFF"


def get_status_color(status):
    if status == "Completed":
        return "#50C878"
    elif status == "Pending":
        return "bold red"
    else:
        return "#FFFFFF"


def sanitize_path(path):
    if path[-1] == "/":
        echo(
            style(
                text="Error: Path is a directory, please provide a file path.",
                fg="red",
            ),
        )
        return False
    if not os.path.exists(os.path.dirname(path)):
        echo(
            style(
                text="Error: The directory where you are trying to store the file in does not exist.",
                fg="red",
            ),
        )
        return False
    return True


def treeify(tasks, title="Tasks"):

    root = {"data": title, "children": []}
    existing_node = {}

    for task in tasks:
        if task["parent_id"] in (None, -1):
            if existing_node.get(task["id"], False):
                existing_node[task["id"]]["data"] = task
            else:
                existing_node[task["id"]] = {"data": task, "children": []}
            if task["parent_id"] == -1:
                root = existing_node[task["id"]]
            else:
                root["children"].append(existing_node[task["id"]])
        else:

            if task["id"] in existing_node:
                existing_node[task["id"]]["data"] = task
            else:
                existing_node[task["id"]] = {"data": task, "children": []}

            if task["parent_id"] not in existing_node:
                parent_task = {
                    "data": {},
                    "children": [],
                }  # temparary node for adding child
                existing_node[task["parent_id"]] = parent_task

            existing_node[task["parent_id"]]["children"].append(
                existing_node[task["id"]],
            )
    return root



from setuptools import find_packages
from setuptools import setup

from app.__version__ import VERSION

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
with open("requirements.txt", "r", encoding="utf-8") as fh:
    requirements = fh.read()


setup(
    name="special-octo-robot",
    version=VERSION,
    author="Jahan Chaware",
    author_email="sg550js@gmail.com",
    license="GNU GENERAL PUBLIC LICENSE",
    description="Creating my own version of what a CLI Task manager and To-do-list would look like",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/geekNero/special-octo-robot.git",
    py_modules=["devcord", "app"],
    packages=find_packages(),
    install_requires=[requirements],
    python_requires=">=3.10",
    platforms=["Windows", "MacOS X", "Linux"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX :: Linux",
    ],
    entry_points={
        "console_scripts": [
            "devcord=devcord:cli",
        ],
    },
)
